package de.hhu.ba.yoshikoWrapper.tasks;

import java.util.HashMap;

import org.cytoscape.model.CyEdge;
import org.cytoscape.model.CyNetwork;
import org.cytoscape.model.CyNode;
import org.cytoscape.model.CyEdge.Type;
import org.cytoscape.model.subnetwork.CySubNetwork;
import org.cytoscape.view.layout.CyLayoutAlgorithm;
import org.cytoscape.view.model.CyNetworkView;
import org.cytoscape.work.AbstractTask;
import org.cytoscape.work.Task;
import org.cytoscape.work.TaskIterator;
import org.cytoscape.work.TaskMonitor;

import de.hhu.ba.yoshikoWrapper.core.CyCore;
import de.hhu.ba.yoshikoWrapper.core.LocalizationManager;
import de.hhu.ba.yoshikoWrapper.cytoUtil.StyleManager;
import de.hhu.ba.yoshikoWrapper.graphModel.YoshikoCluster;
import de.hhu.ba.yoshikoWrapper.graphModel.YoshikoSolution;

public class CreateMetaGraphTask extends AbstractTask {

	private final YoshikoSolution solution;

	public CreateMetaGraphTask(YoshikoSolution s) {
		this.solution = s;
	}

	@Override
	public void run(TaskMonitor taskMonitor) throws Exception {
		CyNetwork metaGraph = CyCore.networkFactory.createNetwork();

		metaGraph.getRow(metaGraph).set(CyNetwork.NAME, LocalizationManager.get("metaGraph"));

		metaGraph.getDefaultNodeTable().createColumn("clusterSize", Integer.class, false);
		metaGraph.getDefaultEdgeTable().createColumn("edgeStrength", Integer.class, false);

		CyLayoutAlgorithm layout = CyCore.layoutAlgorithmManager.getDefaultLayout();


		//Map Cluster to Nodes for further reference
		HashMap<YoshikoCluster,CyNode> map = new HashMap<YoshikoCluster,CyNode>();


		//Add nodes
		for (YoshikoCluster c: solution.getClusters()) {

			CyNode clusterNode = metaGraph.addNode();
			CySubNetwork subnet = c.getSubNetwork();

			taskMonitor.setStatusMessage(LocalizationManager.get("status_createCV")+" "+(c.getID()+1));
			CyCore.networkManager.addNetwork(subnet);
			//Create network view and register it
			CyNetworkView subnetView = CyCore.networkViewFactory.createNetworkView(subnet);
			//layout cluster
			CyCore.dialogTaskManager.execute(
				layout.createTaskIterator(
						subnetView,
						layout.getDefaultLayoutContext(),
						CyLayoutAlgorithm.ALL_NODE_VIEWS,
						null
				)
			);

			StyleManager.style(subnetView,CyCore.visualMappingManager.getCurrentVisualStyle());

			CyCore.networkViewManager.addNetworkView(subnetView);

			clusterNode.setNetworkPointer(subnet);
			//Set node attributes
			metaGraph.getRow(clusterNode).set("name", LocalizationManager.get("cluster")+" "+c.getID());
			metaGraph.getRow(clusterNode).set(StyleManager.CLUSTERSIZE_COLUMN_NAME,c.getSize());
			map.put(c, clusterNode);
		}
		taskMonitor.setStatusMessage(LocalizationManager.get("metaGraph_edges"));
		//Add edges (O(|C|^2*|V|^2) can maybe be improved? //TODO
		for (YoshikoCluster c1:solution.getClusters()) {
			for (YoshikoCluster c2:solution.getClusters()) {
				if (
						metaGraph.containsEdge(map.get(c1),map.get(c2))
						|| c1 == c2
						) {
					continue;
				}
				for (CyNode c1n : c1.getSubNetwork().getNodeList()) {
					for (CyNode c2n : c2.getSubNetwork().getNodeList()) {
						if (solution.getOriginalGraph().containsEdge(c1n, c2n) || solution.getOriginalGraph().containsEdge(c2n, c1n)) {
							if (!metaGraph.containsEdge(map.get(c1), map.get(c2))){
								CyEdge edge = metaGraph.addEdge(map.get(c1), map.get(c2), false);
								metaGraph.getRow(edge).set(StyleManager.EDGESTRENGTH_COLUMN_NAME,1);
							}
							else {
								CyEdge edge = metaGraph.getConnectingEdgeList(map.get(c1), map.get(c2), Type.ANY).get(0);
								metaGraph.getRow(edge).set(StyleManager.EDGESTRENGTH_COLUMN_NAME,
								metaGraph.getRow(edge).get(StyleManager.EDGESTRENGTH_COLUMN_NAME, Integer.class)+1
										);
							}
						}
					}
				}
			}
		}

		CyNetworkView view = CyCore.networkViewFactory.createNetworkView(metaGraph);

		//Layout and style solution

		TaskIterator it = layout.createTaskIterator(
				view,
				layout.getDefaultLayoutContext(),
				CyLayoutAlgorithm.ALL_NODE_VIEWS,
				null
		);

		it.append(new Task() {

			@Override
			public void run(TaskMonitor taskMonitor) throws Exception {
				StyleManager.styleWithMapping(view, CyCore.visualMappingManager.getCurrentVisualStyle());
			}

			@Override
			public void cancel() {}

		});

		CyCore.dialogTaskManager.execute(
			it
		);

		CyCore.networkManager.addNetwork(metaGraph);
		CyCore.networkViewManager.addNetworkView(
			view
		);

		view.updateView();

		CyCore.cy.setCurrentNetworkView(view);
	}

}
